var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx } from "react/jsx-runtime";
/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { useCallback, useRef, useEffect } from "react";
import { Box } from "@theme-ui/components";
import { getPosition } from "../../utils/position";
import Modal from "react-modal";
import ReactDOM from "react-dom";
import { getPopupContainer, getToolbarElement } from "../../toolbar/utils/dom";
import { useIsMobile, useToolbarStore } from "../../toolbar/stores/toolbar-store";
import { ResponsivePresenter } from "../responsive";
import { ThemeProvider } from "../theme-provider";
function _PopupPresenter(props) {
    const { isOpen, position, onClose, blocking = true, focusOnRender = true, children } = props;
    const isMobile = useIsMobile();
    const contentRef = useRef();
    const observerRef = useRef();
    const repositionPopup = useCallback((position) => {
        if (!contentRef.current || !position)
            return;
        const popup = contentRef.current;
        const popupPosition = getPosition(popup, position);
        popup.style.top = popupPosition.top + "px";
        popup.style.left = popupPosition.left + "px";
    }, []);
    useEffect(() => {
        repositionPopup(position);
    }, [repositionPopup, position]);
    useEffect(() => {
        function onWindowResize() {
            repositionPopup(position);
        }
        window.addEventListener("resize", onWindowResize);
        return () => {
            window.removeEventListener("resize", onWindowResize);
        };
    }, [repositionPopup, position]);
    const attachMoveHandlers = useCallback(() => {
        if (!contentRef.current || !isOpen)
            return;
        const movableBar = contentRef.current.querySelector(".movable");
        if (!movableBar)
            return;
        const popup = contentRef.current;
        const offset = { x: 0, y: 0 };
        function mouseDown(e) {
            offset.x = e.clientX - popup.offsetLeft;
            offset.y = e.clientY - popup.offsetTop;
            window.addEventListener("mousemove", mouseMove);
        }
        function mouseMove(e) {
            if (!e.buttons)
                mouseUp();
            const top = e.clientY - offset.y;
            const left = e.clientX - offset.x;
            requestAnimationFrame(() => {
                popup.style.top = top + "px";
                popup.style.left = left + "px";
            });
        }
        function mouseUp() {
            window.removeEventListener("mousemove", mouseMove);
        }
        movableBar.addEventListener("mousedown", mouseDown);
        window.addEventListener("mouseup", mouseUp);
    }, [isOpen]);
    const handleResize = useCallback(() => {
        const popup = contentRef.current;
        if (!popup)
            return;
        let oldHeight = popup.offsetHeight;
        observerRef.current = new ResizeObserver(() => {
            if (isMobile) {
                repositionPopup(position);
            }
            else {
                const { height, y } = popup.getBoundingClientRect();
                const delta = height - oldHeight;
                if (delta > 0) {
                    // means the new size is bigger so we need to adjust the position
                    // if required. We only do this in case the newly resized popup
                    // is going out of the window.
                    const windowHeight = document.body.clientHeight - 20;
                    if (y + height > windowHeight) {
                        popup.style.top = windowHeight - height + "px";
                    }
                }
                oldHeight = height;
            }
        });
        observerRef.current.observe(popup, { box: "border-box" });
    }, [repositionPopup, position, isMobile]);
    return (_jsx(Modal, Object.assign({ contentRef: (ref) => (contentRef.current = ref), className: "popup-presenter", role: "menu", isOpen: isOpen, appElement: document.body, shouldCloseOnEsc: true, shouldReturnFocusAfterClose: true, shouldCloseOnOverlayClick: true, shouldFocusAfterRender: focusOnRender, ariaHideApp: blocking, preventScroll: blocking, onRequestClose: onClose, portalClassName: "popup-presenter-portal", onAfterOpen: (obj) => {
            if (!obj || !position)
                return;
            repositionPopup(position);
            handleResize();
            attachMoveHandlers();
        }, onAfterClose: () => { var _a; return (_a = observerRef.current) === null || _a === void 0 ? void 0 : _a.disconnect(); }, overlayElement: (props, contentEl) => {
            return (_jsx(Box, Object.assign({}, props, { style: Object.assign(Object.assign({}, props.style), { position: !blocking ? "initial" : "fixed", zIndex: 1000, backgroundColor: !blocking ? "transparent" : "unset" }) }, { children: contentEl })));
        }, contentElement: (props, children) => (_jsx(Box, Object.assign({}, props, { style: {}, 
            // TODO onMouseDown={(e) => {
            //   console.log(e);
            // }}
            sx: {
                top: 0,
                left: 0,
                right: 0,
                bottom: 0,
                display: "flex",
                width: "fit-content",
                height: "fit-content",
                position: "fixed",
                backgroundColor: undefined,
                padding: 0,
                zIndex: 999,
                outline: 0,
                isolation: "isolate"
            } }, { children: children }))), style: {
            content: {},
            overlay: {
                zIndex: 999,
                background: "transparent"
            }
        } }, { children: children })));
}
export function PopupPresenter(props) {
    // HACK: we don't want to render the popup presenter for no reason
    // including it's effects etc. so we just wrap it and return null
    // if the popup is closed.
    if (!props.isOpen)
        return null;
    return _jsx(_PopupPresenter, Object.assign({}, props));
}
export function PopupWrapper(props) {
    const { id, position, children, autoCloseOnUnmount } = props, presenterProps = __rest(props, ["id", "position", "children", "autoCloseOnUnmount"]);
    const { closePopup, isPopupOpen } = usePopupHandler(props);
    useEffect(() => {
        if (!autoCloseOnUnmount)
            return;
        return () => {
            closePopup(id);
        };
    }, [autoCloseOnUnmount, id, closePopup]);
    return (_jsx(PopupPresenter, Object.assign({ onClose: () => closePopup(id), position: position, blocking: true, focusOnRender: true }, presenterProps, { isOpen: isPopupOpen }, { children: _jsx(Box, Object.assign({ sx: {
                boxShadow: "menu",
                borderRadius: "default",
                overflow: "hidden"
            } }, { children: children })) }), id));
}
export function usePopupHandler(options) {
    const { isOpen, id, onClosed, group } = options;
    const openedPopups = useToolbarStore((store) => store.openedPopups);
    const openPopup = useToolbarStore((store) => store.openPopup);
    const closePopup = useToolbarStore((store) => store.closePopup);
    const closePopupGroup = useToolbarStore((store) => store.closePopupGroup);
    const isPopupOpen = typeof openedPopups[id] === "object";
    const isPopupDefined = typeof openedPopups[id] !== "undefined";
    useEffect(() => {
        if (isOpen)
            openPopup({ id, group });
        else
            closePopup(id);
    }, [isOpen, closePopup, openPopup, id, group]);
    useEffect(() => {
        // we don't want to close the popup just when it is about to open.
        if (!isPopupOpen && isPopupDefined)
            onClosed === null || onClosed === void 0 ? void 0 : onClosed();
    }, [isPopupOpen, isPopupDefined]);
    useEffect(() => {
        // if another popup in the same group is open, close it.
        if (isPopupOpen) {
            closePopupGroup(group, [id]);
        }
    }, [onClosed, isPopupOpen, closePopupGroup, id, group]);
    return { isPopupOpen, closePopup };
}
export function showPopup(options) {
    const { popup } = options, props = __rest(options, ["popup"]);
    function hide() {
        ReactDOM.unmountComponentAtNode(getPopupContainer());
    }
    ReactDOM.render(_jsx(ThemeProvider, { children: _jsx(ResponsivePresenter, Object.assign({ isOpen: true, position: {
                target: getToolbarElement(),
                isTargetAbsolute: true,
                location: "below",
                align: "end",
                yOffset: 10
            }, blocking: true, focusOnRender: true }, props, { onClose: () => {
                var _a;
                hide();
                (_a = props.onClose) === null || _a === void 0 ? void 0 : _a.call(props);
            } }, { children: popup(hide) })) }), getPopupContainer());
    return hide;
}
