import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { useRef, useState } from "react";
import { Flex, Text } from "@theme-ui/components";
import Languages from "./languages.json";
import { Input } from "@theme-ui/components";
import { Icon } from "../../toolbar/components/icon";
import { Icons } from "../../toolbar/icons";
import { ResponsivePresenter } from "../../components/responsive";
import { Popup } from "../../toolbar/components/popup";
import { Button } from "../../components/button";
export function CodeblockComponent(props) {
    const { editor, updateAttributes, node, forwardRef } = props;
    const { language, indentLength, indentType, caretPosition } = node.attrs;
    const [isOpen, setIsOpen] = useState(false);
    // const [caretPosition, setCaretPosition] = useState<CaretPosition>();
    const toolbarRef = useRef(null);
    const languageDefinition = Languages.find((l) => { var _a; return l.filename === language || ((_a = l.alias) === null || _a === void 0 ? void 0 : _a.some((a) => a === language)); });
    return (_jsxs(_Fragment, { children: [_jsxs(Flex, Object.assign({ sx: {
                    flexDirection: "column",
                    borderRadius: "default",
                    overflow: "hidden"
                } }, { children: [_jsx(Text, { ref: forwardRef, as: "pre", autoCorrect: "off", autoCapitalize: "none", css: {}, sx: {
                            "div, span.token, span.line-number-widget, span.line-number::before": {
                                fontFamily: "monospace",
                                fontSize: "code",
                                whiteSpace: "pre",
                                tabSize: 1
                            },
                            position: "relative",
                            lineHeight: "20px",
                            bg: "codeBg",
                            color: "static",
                            overflowX: "auto",
                            display: "flex",
                            px: 2,
                            pt: 2,
                            pb: 1
                        }, spellCheck: false }), _jsxs(Flex, Object.assign({ ref: toolbarRef, contentEditable: false, sx: {
                            bg: "codeBg",
                            alignItems: "center",
                            justifyContent: "flex-end",
                            borderTop: "1px solid var(--codeBorder)"
                        } }, { children: [caretPosition ? (_jsxs(Text, Object.assign({ variant: "subBody", sx: { mr: 2, color: "codeFg" } }, { children: ["Line ", caretPosition.line, ", Column ", caretPosition.column, " ", caretPosition.selected
                                        ? `(${caretPosition.selected} selected)`
                                        : ""] }))) : null, _jsx(Button, Object.assign({ variant: "icon", sx: {
                                    p: 1,
                                    mr: 1,
                                    opacity: "1 !important",
                                    ":hover": { bg: "codeSelection" }
                                }, title: "Toggle indentation mode", disabled: !editor.isEditable, onClick: () => {
                                    if (!editor.isEditable)
                                        return;
                                    editor.commands.changeCodeBlockIndentation({
                                        type: indentType === "space" ? "tab" : "space",
                                        amount: indentLength
                                    });
                                } }, { children: _jsxs(Text, Object.assign({ variant: "subBody", sx: { color: "codeFg" } }, { children: [indentType === "space" ? "Spaces" : "Tabs", ": ", indentLength] })) })), _jsx(Button, Object.assign({ variant: "icon", sx: {
                                    opacity: "1 !important",
                                    p: 1,
                                    mr: 1,
                                    bg: isOpen ? "codeSelection" : "transparent",
                                    ":hover": { bg: "codeSelection" }
                                }, disabled: !editor.isEditable, onClick: () => {
                                    if (!editor.isEditable)
                                        return;
                                    setIsOpen(true);
                                }, title: "Change language" }, { children: _jsx(Text, Object.assign({ variant: "subBody", spellCheck: false, sx: { color: "codeFg" } }, { children: (languageDefinition === null || languageDefinition === void 0 ? void 0 : languageDefinition.title) || "Plaintext" })) }))] }))] })), _jsx(ResponsivePresenter, Object.assign({ isOpen: isOpen, onClose: () => {
                    setIsOpen(false);
                    // NOTE: for some reason the language selection action sheet
                    // does not return focus to the last focused position after
                    // closing. We have to set focusOnRender=false & manually
                    // restore focus. I think this has something to do with custom
                    // node views.
                    // TRY: perhaps use SelectionBasedReactNodeView?
                    editor.commands.focus();
                }, focusOnRender: false, mobile: "sheet", desktop: "menu", position: {
                    target: toolbarRef.current || undefined,
                    align: "end",
                    isTargetAbsolute: true,
                    location: "top",
                    yOffset: 5
                }, title: "Change code block language" }, { children: _jsx(LanguageSelector, { selectedLanguage: (languageDefinition === null || languageDefinition === void 0 ? void 0 : languageDefinition.filename) || "Plaintext", onLanguageSelected: (language) => {
                        updateAttributes({ language }, { addToHistory: true, preventUpdate: false });
                        setIsOpen(false);
                    }, onClose: () => setIsOpen(false) }) }))] }));
}
function LanguageSelector(props) {
    const { onLanguageSelected, selectedLanguage, onClose } = props;
    const [languages, setLanguages] = useState(Languages);
    return (_jsx(Popup, Object.assign({ title: "Select language", onClose: onClose }, { children: _jsxs(Flex, Object.assign({ sx: {
                flexDirection: "column",
                height: 200,
                width: ["auto", 300],
                overflowY: "auto",
                bg: "background"
            } }, { children: [_jsx(Input, { onFocus: () => {
                        console.log("EHLLO!");
                    }, autoFocus: true, placeholder: "Search languages", sx: {
                        width: "auto",
                        position: "sticky",
                        top: 0,
                        bg: "background",
                        mx: 2,
                        p: "7px",
                        zIndex: 999
                    }, onChange: (e) => {
                        if (!e.target.value)
                            return setLanguages(Languages);
                        const query = e.target.value.toLowerCase();
                        setLanguages(Languages.filter((lang) => {
                            var _a;
                            return (lang.title.toLowerCase().indexOf(query) > -1 ||
                                ((_a = lang.alias) === null || _a === void 0 ? void 0 : _a.some((alias) => alias.toLowerCase().indexOf(query) > -1)));
                        }));
                    } }), _jsx(Flex, Object.assign({ sx: {
                        flexDirection: "column",
                        pt: 1,
                        mt: 1
                    } }, { children: languages.map((lang) => (_jsxs(Button, Object.assign({ variant: "menuitem", sx: {
                            textAlign: "left",
                            py: 1,
                            display: "flex",
                            justifyContent: "space-between",
                            alignItems: "center"
                        }, onClick: () => onLanguageSelected(lang.filename) }, { children: [_jsx(Text, Object.assign({ variant: "body" }, { children: lang.title })), selectedLanguage === lang.filename ? (_jsx(Icon, { path: Icons.check, size: "small" })) : lang.alias ? (_jsx(Text, Object.assign({ variant: "subBody", sx: { fontSize: "10px" } }, { children: lang.alias.slice(0, 3).join(", ") }))) : null] }), lang.title))) }))] })) })));
}
