var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { Box, Button, Flex, Image, Text } from "@theme-ui/components";
import { useEffect, useRef, useState } from "react";
import { DesktopOnly } from "../../components/responsive";
import { Icon } from "../../toolbar/components/icon";
import { Icons } from "../../toolbar/icons";
import { ToolbarGroup } from "../../toolbar/components/toolbar-group";
import { useIsMobile, useToolbarStore } from "../../toolbar/stores/toolbar-store";
import { Resizer } from "../../components/resizer";
import { downloadImage, isDataUrl, toBlobURL, toDataURL } from "../../utils/downloader";
import { m, LazyMotion, domAnimation } from "framer-motion";
export const AnimatedImage = m(Image);
export function ImageComponent(props) {
    const { editor, node, selected } = props;
    const isMobile = useIsMobile();
    const { dataurl, src, alt, title, width, height, textDirection, hash, aspectRatio } = node.attrs;
    const float = isMobile ? false : node.attrs.float;
    let align = node.attrs.align;
    if (!align)
        align = textDirection ? "right" : "left";
    const imageRef = useRef(null);
    const [error, setError] = useState();
    const [source, setSource] = useState();
    const downloadOptions = useToolbarStore((store) => store.downloadOptions);
    useEffect(() => {
        (() => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!src && !dataurl)
                return;
            try {
                if (dataurl)
                    setSource(yield toBlobURL(dataurl));
                else if (isDataUrl(src))
                    setSource(yield toBlobURL(src));
                else {
                    const { url, size, blob, type } = yield downloadImage(src, downloadOptions);
                    setSource(url);
                    (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.updateImage({ src }, { src: yield toDataURL(blob), size, type });
                }
            }
            catch (e) {
                console.error(e);
                if (e instanceof Error)
                    setError(e.message);
            }
        }))();
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [src, dataurl, imageRef, downloadOptions]);
    const relativeHeight = aspectRatio
        ? editor.view.dom.clientWidth / aspectRatio
        : undefined;
    return (_jsx(_Fragment, { children: _jsx(Box, Object.assign({ sx: {
                display: float ? "inline" : "flex",
                ml: float ? (align === "right" ? 2 : 0) : 0,
                mr: float ? (align === "left" ? 2 : 0) : 0,
                float: float ? align : "none",
                justifyContent: float
                    ? "stretch"
                    : align === "center"
                        ? "center"
                        : align === "left"
                            ? "start"
                            : "end",
                ":hover .drag-handle, :active .drag-handle": {
                    opacity: 1
                }
            } }, { children: _jsx(LazyMotion, Object.assign({ features: domAnimation }, { children: !source || error ? (_jsxs(Flex, Object.assign({ sx: {
                        width: width || "100%",
                        height: height || relativeHeight || "100%",
                        maxWidth: "100%",
                        minWidth: 135,
                        bg: "bgSecondary",
                        border: selected
                            ? "2px solid var(--primary)"
                            : "2px solid transparent",
                        borderRadius: "default",
                        flexDirection: "column",
                        alignItems: "center",
                        justifyContent: "center",
                        mt: 1,
                        py: 50
                    } }, { children: [_jsx(Icon, { path: error
                                ? Icons.imageFailed
                                : isDownloadable(source, src)
                                    ? Icons.imageDownload
                                    : Icons.image, size: width ? width * 0.2 : 72, color: "gray" }), _jsx(Text, Object.assign({ as: "span", variant: "subBody", sx: {
                                display: "inline-block",
                                textDecoration: "none",
                                textAlign: "center",
                                mx: 2,
                                mt: 2
                            } }, { children: error
                                ? `There was an error loading the image: ${error}`
                                : isDownloadable(source, src)
                                    ? `Downloading image from ${getHostname(src)}`
                                    : "" })), error ? (_jsx(Button, Object.assign({ variant: "secondary", sx: { mt: 1 }, onClick: () => {
                                setSource(src);
                                setError(undefined);
                            } }, { children: "Skip downloading (unsafe)" }))) : null] }))) : (_jsxs(Resizer, Object.assign({ style: { marginTop: 5 }, editor: editor, selected: selected, width: width, onResize: (width, height) => {
                        editor.commands.setImageSize({ width, height });
                    } }, { children: [_jsx(DesktopOnly, { children: selected && (_jsx(Flex, Object.assign({ sx: { position: "relative", justifyContent: "end" } }, { children: _jsx(Flex, Object.assign({ sx: {
                                        position: "absolute",
                                        top: -40,
                                        mb: 2,
                                        alignItems: "end"
                                    } }, { children: _jsx(ToolbarGroup, { editor: editor, tools: [
                                            hash ? "downloadAttachment" : "none",
                                            "imageAlignLeft",
                                            float ? "none" : "imageAlignCenter",
                                            "imageAlignRight",
                                            "imageProperties"
                                        ], sx: {
                                            boxShadow: "menu",
                                            borderRadius: "default",
                                            bg: "background"
                                        } }) })) }))) }), selected && (_jsx(Icon, { className: "drag-handle", "data-drag-handle": true, draggable: true, path: Icons.dragHandle, color: "black", sx: {
                                cursor: "grab",
                                position: "absolute",
                                top: 1,
                                left: 1,
                                zIndex: 999
                            } })), _jsx(AnimatedImage, { initial: { opacity: 0 }, animate: { opacity: 1 }, transition: { duration: 0.5, ease: "easeIn" }, "data-drag-image": true, ref: imageRef, alt: alt, src: source, title: title, sx: {
                                width: editor.isEditable ? "100%" : width,
                                height: editor.isEditable ? "100%" : height,
                                border: selected
                                    ? "2px solid var(--primary)"
                                    : "2px solid transparent",
                                borderRadius: "default"
                            }, onLoad: (e) => {
                                var _a;
                                const { clientHeight, clientWidth } = e.currentTarget;
                                if (!height && !width && !aspectRatio) {
                                    (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.updateImage({ src, hash }, { aspectRatio: clientWidth / clientHeight });
                                }
                            } })] }))) })) })) }));
}
function getHostname(src) {
    if (!src)
        return null;
    return new URL(src).hostname;
}
function isDownloadable(source, src) {
    return !source && src && !isDataUrl(src);
}
