import { PluginKey } from "prosemirror-state";
export interface Listeners<T> {
    [name: string]: Set<Listener<T>>;
}
export declare type Listener<T = never> = (data: T) => void;
export declare type Dispatch<T = never> = (eventName: PluginKey | string, data: T) => void;
export declare class EventDispatcher<T = never> {
    private listeners;
    on(event: string, cb: Listener<T>): void;
    off(event: string, cb: Listener<T>): void;
    emit(event: string, data: T): void;
    destroy(): void;
}
/**
 * Creates a dispatch function that can be called inside ProseMirror Plugin
 * to notify listeners about that plugin's state change.
 */
export declare function createDispatch<T>(eventDispatcher: EventDispatcher<T>): Dispatch<T>;
