import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { Input } from "@theme-ui/components";
import { useCallback, useEffect, useRef, useState } from "react";
import { Flex, Text } from "@theme-ui/components";
import { ToolButton } from "../components/tool-button";
export function SearchReplacePopup(props) {
    const { editor } = props;
    const { selectedText, results, selectedIndex, focusNonce } = editor.storage
        .searchreplace;
    const [isReplacing, setIsReplacing] = useState(false);
    const [isExpanded, setIsExpanded] = useState(false);
    const [matchCase, setMatchCase] = useState(false);
    const [matchWholeWord, setMatchWholeWord] = useState(false);
    const [enableRegex, setEnableRegex] = useState(false);
    const replaceText = useRef("");
    const searchInputRef = useRef(null);
    const search = useCallback((term) => {
        var _a;
        (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.search(term, {
            matchCase,
            enableRegex,
            matchWholeWord
        });
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [matchCase, enableRegex, matchWholeWord]);
    useEffect(() => {
        if (!searchInputRef.current)
            return;
        search(searchInputRef.current.value);
    }, [search, matchCase, matchWholeWord, enableRegex]);
    useEffect(() => {
        if (selectedText) {
            if (searchInputRef.current) {
                const input = searchInputRef.current;
                setTimeout(() => {
                    input.value = selectedText;
                    input.focus();
                }, 0);
            }
            search(selectedText);
        }
    }, [selectedText, search]);
    useEffect(() => {
        if (searchInputRef.current) {
            const input = searchInputRef.current;
            setTimeout(() => {
                input.focus();
            }, 0);
        }
    }, [focusNonce]);
    return (_jsx(Flex, Object.assign({ sx: {
            p: 1,
            bg: "background",
            flexDirection: "column",
            boxShadow: ["none", "menu"],
            borderRadius: [0, "default"]
        } }, { children: _jsxs(Flex, { children: [_jsxs(Flex, Object.assign({ sx: { flexDirection: "column", flex: 1, width: ["auto", 300], mr: 1 } }, { children: [_jsxs(Flex, Object.assign({ sx: {
                                flex: 1,
                                position: "relative",
                                alignItems: "center",
                                outline: "1px solid var(--border)",
                                borderRadius: "default",
                                p: 1,
                                py: 0,
                                ":focus-within": {
                                    outlineColor: "primary",
                                    outlineWidth: "1.8px"
                                },
                                ":hover": {
                                    outlineColor: "primary"
                                }
                            } }, { children: [_jsx(Input, { variant: "clean", defaultValue: selectedText, ref: searchInputRef, autoFocus: true, placeholder: "Find", sx: { p: 0 }, onChange: (e) => {
                                        search(e.target.value);
                                    }, onKeyDown: (e) => {
                                        var _a;
                                        if (e.key === "Enter") {
                                            (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.moveToNextResult();
                                        }
                                    } }), _jsxs(Flex, Object.assign({ sx: {
                                        flexShrink: 0,
                                        mr: 0,
                                        alignItems: "center"
                                    } }, { children: [_jsx(ToolButton, { sx: {
                                                mr: 0
                                            }, toggled: isExpanded, title: "Expand", id: "expand", icon: isExpanded ? "chevronRight" : "chevronLeft", onClick: () => setIsExpanded((s) => !s), iconSize: "medium" }), isExpanded && (_jsxs(_Fragment, { children: [_jsx(ToolButton, { sx: {
                                                        mr: 0
                                                    }, toggled: matchCase, title: "Match case", id: "matchCase", icon: "caseSensitive", onClick: () => setMatchCase((s) => !s), iconSize: "medium" }), _jsx(ToolButton, { sx: {
                                                        mr: 0
                                                    }, toggled: matchWholeWord, title: "Match whole word", id: "matchWholeWord", icon: "wholeWord", onClick: () => setMatchWholeWord((s) => !s), iconSize: "medium" }), _jsx(ToolButton, { sx: {
                                                        mr: 0
                                                    }, toggled: enableRegex, title: "Enable regex", id: "enableRegex", icon: "regex", onClick: () => setEnableRegex((s) => !s), iconSize: "medium" })] })), _jsx(Text, Object.assign({ variant: "subBody", sx: {
                                                flexShrink: 0,
                                                borderLeft: "1px solid var(--border)",
                                                color: "fontTertiary",
                                                px: 1
                                            } }, { children: results ? `${selectedIndex + 1}/${results.length}` : "" }))] }))] })), isReplacing && (_jsx(Input, { sx: { mt: 1, p: "7px" }, placeholder: "Replace", onChange: (e) => (replaceText.current = e.target.value) }))] })), _jsxs(Flex, Object.assign({ sx: { flexDirection: "column" } }, { children: [_jsxs(Flex, Object.assign({ sx: { alignItems: "center", height: "33.2px" } }, { children: [_jsx(ToolButton, { toggled: isReplacing, title: "Toggle replace", id: "toggleReplace", icon: "replace", onClick: () => setIsReplacing((s) => !s), sx: { mr: 0 }, iconSize: "big" }), _jsx(ToolButton, { toggled: false, title: "Previous match", id: "previousMatch", icon: "previousMatch", onClick: () => { var _a; return (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.moveToPreviousResult(); }, sx: { mr: 0 }, iconSize: "big" }), _jsx(ToolButton, { toggled: false, title: "Next match", id: "nextMatch", icon: "nextMatch", onClick: () => { var _a; return (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.moveToNextResult(); }, sx: { mr: 0 }, iconSize: "big" }), _jsx(ToolButton, { toggled: false, title: "Close", id: "close", icon: "close", onClick: () => { var _a; return (_a = editor.current) === null || _a === void 0 ? void 0 : _a.chain().focus().endSearch().run(); }, sx: { mr: 0 }, iconSize: "big" })] })), isReplacing && (_jsxs(Flex, Object.assign({ sx: { alignItems: "center", height: "33.2px", mt: 1 } }, { children: [_jsx(ToolButton, { toggled: false, title: "Replace", id: "replace", icon: "replaceOne", onClick: () => { var _a; return (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.replace(replaceText.current); }, sx: { mr: 0 }, iconSize: 18 }), _jsx(ToolButton, { toggled: false, title: "Replace all", id: "replaceAll", icon: "replaceAll", onClick: () => { var _a; return (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.replaceAll(replaceText.current); }, sx: { mr: 0 }, iconSize: 18 })] })))] }))] }) })));
}
