var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { ToolButton } from "../components/tool-button";
import { useCallback, useRef, useState } from "react";
import { ResponsivePresenter } from "../../components/responsive";
import { LinkPopup } from "../popups/link-popup";
import { useToolbarLocation } from "../stores/toolbar-store";
import { MoreTools } from "../components/more-tools";
import { useRefValue } from "../../hooks/use-ref-value";
import { findMark, selectionToOffset } from "../../utils/prosemirror";
import { TextSelection } from "prosemirror-state";
import { Flex, Link } from "@theme-ui/components";
import { ImageNode } from "../../extensions/image";
export function LinkSettings(props) {
    const { editor } = props;
    const isBottom = useToolbarLocation() === "bottom";
    if (!editor.isActive("link") || !isBottom)
        return null;
    return (_jsx(MoreTools, Object.assign({}, props, { autoOpen: true, autoCloseOnUnmount: true, popupId: "linkSettings", tools: ["openLink", "editLink", "removeLink"] })));
}
export function AddLink(props) {
    const { editor } = props;
    const isActive = props.editor.isActive("link");
    const onDone = useCallback((link) => {
        var _a, _b;
        const { href, text, isImage } = link;
        if (!href)
            return;
        let commandChain = (_a = editor.current) === null || _a === void 0 ? void 0 : _a.chain().focus();
        if (!commandChain)
            return;
        const isSelection = !((_b = editor.current) === null || _b === void 0 ? void 0 : _b.state.selection.empty);
        commandChain = commandChain
            .extendMarkRange("link")
            .toggleLink({ href, target: "_blank" });
        if (!isImage)
            commandChain = commandChain.insertContent(text || href);
        commandChain = commandChain.focus();
        if (!isSelection && !isImage)
            commandChain = commandChain.unsetMark("link").insertContent(" ");
        commandChain.run();
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    if (isActive)
        return _jsx(EditLink, Object.assign({}, props, { icon: "linkEdit" }));
    return (_jsx(LinkTool, Object.assign({}, props, { onDone: onDone, onClick: () => {
            var _a;
            if (!editor.current)
                return;
            const { state } = editor.current;
            const { from, to } = state.selection;
            const isImage = ((_a = state.doc.nodeAt(from)) === null || _a === void 0 ? void 0 : _a.type.name) === ImageNode.name;
            if (isImage)
                return { isImage };
            const selectedText = state.doc.textBetween(from, to);
            return { text: selectedText };
        } })));
}
export function EditLink(props) {
    var _a;
    const { editor, selectedNode: _selectedNode } = props;
    const selectedNode = useRefValue(_selectedNode || selectionToOffset(editor.state));
    const onDone = useCallback((link) => {
        const { href, text, isImage } = link;
        const { from, node, to } = selectedNode.current;
        if (!href || !editor.current || !node)
            return;
        const mark = findMark(node, "link");
        if (!mark)
            return;
        const selection = editor.current.state.selection;
        let commandChain = editor.current.chain();
        if (!isImage) {
            commandChain = commandChain.command(({ tr }) => {
                tr.removeMark(from, to, mark.type);
                tr.insertText(text || node.textContent, tr.mapping.map(from), tr.mapping.map(to));
                tr.setSelection(TextSelection.create(tr.doc, tr.mapping.map(from), tr.mapping.map(to)));
                return true;
            });
        }
        commandChain
            .extendMarkRange("link")
            .toggleLink({ href, target: "_blank" })
            .command(({ tr }) => {
            tr.setSelection(TextSelection.create(tr.doc, tr.mapping.map(selection.from), tr.mapping.map(selection.to)));
            return true;
        })
            .focus(undefined, { scrollIntoView: true })
            .run();
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    if (!((_a = editor.current) === null || _a === void 0 ? void 0 : _a.isEditable))
        return null;
    return (_jsx(LinkTool, Object.assign({}, props, { isEditing: true, onDone: onDone, onClick: () => {
            const { node } = selectedNode.current;
            if (!node)
                return;
            const selectedText = node.textContent;
            const mark = findMark(node, "link");
            if (!mark)
                return;
            return {
                text: selectedText,
                href: mark.attrs.href,
                isImage: node.type.name === ImageNode.name
            };
        } })));
}
export function RemoveLink(props) {
    var _a;
    const { editor, selectedNode } = props;
    if (!((_a = editor.current) === null || _a === void 0 ? void 0 : _a.isEditable))
        return null;
    return (_jsx(ToolButton, Object.assign({}, props, { toggled: false, onClick: () => {
            var _a, _b;
            if (selectedNode)
                (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.setTextSelection({
                    from: selectedNode.from,
                    to: selectedNode.to
                });
            (_b = editor.current) === null || _b === void 0 ? void 0 : _b.chain().focus().unsetLink().run();
        } })));
}
export function OpenLink(props) {
    const { editor, selectedNode: _selectedNode } = props;
    const selectedNode = useRefValue(_selectedNode || selectionToOffset(editor.state));
    const { node } = selectedNode.current;
    const link = node ? findMark(node, "link") : null;
    if (!link)
        return null;
    const href = link === null || link === void 0 ? void 0 : link.attrs.href;
    return (_jsxs(Flex, Object.assign({ sx: { alignItems: "center" } }, { children: [_jsx(Link, Object.assign({ href: href, onClick: (e) => {
                    e.preventDefault();
                    editor.commands.openLink(href);
                }, target: "_blank", variant: "body", sx: {
                    fontSize: "subBody",
                    mr: 1,
                    maxWidth: [150, 250],
                    overflow: "hidden",
                    textOverflow: "ellipsis",
                    whiteSpace: "nowrap",
                    ":visited": { color: "primary" },
                    ":hover": { color: "primary", opacity: 0.8 }
                } }, { children: href })), _jsx(ToolButton, Object.assign({}, props, { toggled: false, onClick: () => {
                    editor.commands.openLink(href);
                } }))] })));
}
function LinkTool(props) {
    const { isEditing, onClick, onDone, editor } = props, toolProps = __rest(props, ["isEditing", "onClick", "onDone", "editor"]);
    const buttonRef = useRef(null);
    const [isOpen, setIsOpen] = useState(false);
    const [linkDefinition, setLinkDefinition] = useState();
    return (_jsxs(_Fragment, { children: [_jsx(ToolButton, Object.assign({}, toolProps, { buttonRef: buttonRef, onClick: () => {
                    const result = onClick();
                    if (!result)
                        return;
                    setLinkDefinition(result);
                    setIsOpen(true);
                }, toggled: isOpen })), _jsx(ResponsivePresenter, Object.assign({ mobile: "sheet", desktop: "menu", position: {
                    target: buttonRef.current || undefined,
                    isTargetAbsolute: true,
                    location: "below",
                    align: "center",
                    yOffset: 5
                }, title: isEditing ? "Edit link" : "Insert link", isOpen: isOpen, items: [], onClose: () => {
                    var _a;
                    setIsOpen(false);
                    (_a = editor.current) === null || _a === void 0 ? void 0 : _a.commands.focus();
                }, focusOnRender: false }, { children: _jsx(LinkPopup, { link: linkDefinition, isEditing: isEditing, onClose: () => setIsOpen(false), onDone: (link) => {
                        onDone(link);
                        setIsOpen(false);
                    } }) }))] }));
}
