/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
export function getTotalWords(editor) {
    const documentText = editor.state.doc.textBetween(0, editor.state.doc.content.size, "\n", " ");
    return countWords(documentText);
}
export function countWords(str) {
    let count = 0;
    let shouldCount = false;
    let isScript = false;
    for (let i = 0; i < str.length; ++i) {
        const s = str[i];
        if (s === " " ||
            s === "\r" ||
            s === "\n" ||
            s === "*" ||
            s === "/" ||
            s === "&" ||
            (isScript = isCJKChar(s))) {
            if (!shouldCount && !isScript)
                continue;
            ++count;
            shouldCount = false;
        }
        else {
            shouldCount = true;
        }
    }
    if (shouldCount)
        ++count;
    return count;
}
// Taken from: https://en.wikipedia.org/wiki/CJK_Unified_Ideographs
const CJK_UNICODE_RANGES = [
    [19968, 40959],
    [13312, 19903],
    [131072, 173791],
    [173824, 177983],
    [177984, 178207],
    [178208, 183983],
    [183984, 191471],
    [196608, 201551],
    [201552, 205743],
    [63744, 64255],
    [194560, 195103],
    [12032, 12255],
    [11904, 12031],
    [12288, 12351],
    [13056, 13311],
    [65072, 65103] // CJK Compatibility Forms                     FE30-FE4F
];
function isCJKChar(char) {
    const code = char.charCodeAt(0);
    const isIn = CJK_UNICODE_RANGES.some((range) => code >= range[0] && code <= range[1]);
    return isIn;
}
